import json
import requests

from oteapi.models import TransformationConfig

from otelib.apierror import ApiError
from otelib.abstractfilter import AbstractFilter


class Transformation(AbstractFilter):
    """Context class for the Transformation Strategy Interfaces."""

    def create(self, **kwargs):
        """Create a Transformation."""
        data = TransformationConfig(**kwargs)
        response = requests.post(
            f'{self.url}{self.settings.prefix}/transformation',
            data=json.dumps(data.dict())
        )
        if response.status_code != 200:
            raise ApiError(
                f'Cannot create transformation: {response.status_code}'
            )
        self.data = json.loads(response.text)
        self.id = self.data.pop('transformation_id')

    def fetch(self, session_id):
        """Fetch a specific Transformation with its ID."""
        response = requests.get(
            f'{self.url}{self.settings.prefix}/transformation/{self.id}?'
            f'session_id={session_id}'
        )
        return response.content

    def initialize(self, session_id):
        """Initialize a specific Transformation with its ID."""
        response = requests.post(
            f'{self.url}{self.settings.prefix}/transformation/{self.id}/'
            f'initialize?session_id={session_id}'
        )
        return response.content



class ApiError(Exception):
    """An API Error Exception"""

    def __init__(self, status):
        super().__init__()
        self.status = status

    def __str__(self):
        return f"APIError: status={self.status}"



from .dataresource import DataResource
from .filter import Filter
from .transformation import Transformation
from .pipe import Pipe
from .mapping import Mapping
from .ontotransserver import OntoTransServer


__all__ = ("DataResource", "Filter", "Transformation", "Pipe", "Mapping", "OntoTransServer")



from abc import ABC, abstractmethod
import json
import requests

from otelib.apierror import ApiError
from otelib.pipe import Pipe
from otelib.settings import Settings


class AbstractFilter(ABC):
    """Abstract class for filters."""

    def __init__(self, url):
        """Initiates a filter.
        The `url` is the base URL of the OTEAPI Services.
        """
        self.url = url
        self.settings = Settings()
        self.input_pipe = None

    @abstractmethod
    def create(self, **kwargs):
        """Create a filter.

        It should post the configuration for the created filter.
        """

    @abstractmethod
    def fetch(self, session_id):
        """Returns the result of the current filter.

        This method is called by get() while propagating up the pipeline.

        The `session_id` is id of the session shared by the pipeline.
        """

    @abstractmethod
    def initialize(self, session_id):
        """Initialise the current filter.

        This method is called by get() when propagating down the pipeline.

        The `session_id` is id of the session shared by the pipeline.
        """

    def _set_input(self, input_pipe):
        self.input_pipe = input_pipe

    def __rshift__(self, other):
        """Implements filter concatenation using the `>>` symbol."""
        p = Pipe(self)
        other._set_input(p)
        return other

    def get(self, session_id=None):
        """Executes a pipeline.

        The `session_id` is id of the session shared by the pipeline.

        This will call initialize() and then the get() method on the
        input pipe, which in turn will call the get() method on the
        filter connected to its input and so forth until the beginning
        of the pipeline.

        Finally fetch() is called and its output is returned.
        """
        self.settings = Settings()

        if session_id is None:
            response = requests.post(
                f'{self.url}{self.settings.prefix}/session/', data='{}'
            )
            if response.status_code != 200:
                raise ApiError(
                    f'Cannot create session: {response.status_code}'
                )
            session_id = json.loads(response.text)['session_id']

        self.initialize(session_id)
        if self.input_pipe:
            self.input_pipe.get(session_id)
        return self.fetch(session_id)



import json
import requests

from oteapi.models import ResourceConfig

from otelib.abstractfilter import AbstractFilter
from otelib.apierror import ApiError


class DataResource(AbstractFilter):
    """Context class for the datasource strategy interfaces for managing i/o operations."""

    def create(self, **kwargs):
        """ Create a data resource """
        data = ResourceConfig(**kwargs)
        response = requests.post(
            f'{self.url}{self.settings.prefix}/dataresource/',
            data=json.dumps(data.dict())
        )
        if response.status_code != 200:
            raise ApiError(
                f'Cannot create dataresouce: {response.status_code}'
            )
        self.data = json.loads(response.text)
        self.id = self.data.pop('resource_id')

    def fetch(self, session_id):
        """ Fetch a specific data resource with its ID """
        response = requests.get(
            f'{self.url}{self.settings.prefix}/dataresource/{self.id}'
            f'?session_id={session_id}'
        )
        return response.content

    def initialize(self, session_id):
        """ Initialize a specific data resource with its ID """
        response = requests.post(
            f'{self.url}{self.settings.prefix}/dataresource/{self.id}/'
            f'initialize?session_id={session_id}'
        )
        return response.content



from otelib.dataresource import DataResource
from otelib.transformation import Transformation
from otelib.filter import Filter
from otelib.mapping import Mapping


class OntoTransServer:
    """The OntoTransServer object represent a remote OntoTrans REST API on the network."""

    def __init__(self, url):
        self.url = url

    def create_dataresource(self, **kwargs):
        """Create a new datasource."""
        dr = DataResource(self.url)
        dr.create(**kwargs)
        return dr

    def create_transformation(self, **kwargs):
        """Create new tranformation."""
        tr = Transformation(self.url)
        tr.create(**kwargs)
        return tr

    def create_filter(self, **kwargs):
        """Create new filter."""
        ft = Filter(self.url)
        ft.create(**kwargs)
        return ft

    def create_mapping(self, **kwargs):
        """Create new mapping."""
        mp = Mapping(self.url)
        mp.create(**kwargs)
        return mp



import json
import requests

from oteapi.models import MappingConfig

from otelib.apierror import ApiError
from otelib.abstractfilter import AbstractFilter


class Mapping(AbstractFilter):
    """Context class for the Mapping Strategy Interfaces"""

    def create(self, **kwargs):
        """Create a Mapping."""
        data = MappingConfig(**kwargs)
        response = requests.post(
            f'{self.url}{self.settings.prefix}/mapping',
            data=json.dumps(data.dict())
        )
        if response.status_code != 200:
            raise ApiError(f'Cannot create filter: {response.status_code}')
        self.data = json.loads(response.text)
        self.id = self.data.pop('mapping_id')

    def fetch(self, session_id):
        """Fetch a specific Mapping with its ID."""
        response = requests.get(
            f'{self.url}{self.settings.prefix}/mapping/{self.id}?'
            f'session_id={session_id}'
        )
        return response.content

    def initialize(self, session_id):
        """Initialize a specific Mapping with its ID."""
        response = requests.post(
            f'{self.url}{self.settings.prefix}/mapping/{self.id}/initialize?'
            f'session_id={session_id}'
        )
        return response.content
